#ifndef _SERVERPOOL_H
#define _SERVERPOOL_H 1

#include "server.h"

#include <vector>

// Provides services such as load-balancing between a number of Server instances.
class ServerPool {
public:
	ServerPool(int num_servers);
	~ServerPool();

	// Accessor. Only to be used in rare situations, really.
	// The ServerPool retains ownership.
	Server *get_server(int num) { return &servers[num]; }

	// Picks a server (round-robin) and allocates the given client to it.
	void add_client(int sock);
	void add_client_from_serialized(const ClientProto &client);

	// Adds the given stream to all the servers.
	void add_stream(const std::string &stream_id);
	void add_stream_from_serialized(const StreamProto &stream);

	// Adds the given data to all the servers.
	void set_header(const std::string &stream_id, const std::string &header);
	void add_data(const std::string &stream_id, const char *data, size_t bytes);

	// Starts all the servers.
	void run();

	std::vector<ClientStats> get_client_stats() const;

private:
	Server *servers;
	int num_servers, clients_added;

	ServerPool(const ServerPool &);
};

#endif  // !defined(_SERVERPOOL_H)
